/*
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER, TCSDESIGNER
 * Version: 1.0
 */

using System;
using NUnit.Framework;

namespace TopCoder.File.Statistics
{
    /// <summary>
    /// This class tests the <c>Metric</c> class.
    /// </summary>
    [TestFixture]
    public class TestMetric
    {
        /// <summary>
        /// Three metrics used for testing.
        /// </summary>
        private Metric[] metrics = new Metric[3];

        /// <summary>
        /// Sets up some simple objects used for testing.
        /// </summary>
        [SetUp]
        public void Setup()
        {
            metrics[0] = new Metric("A", "B", "C");
            metrics[1] = new Metric("A", "C", "B");
            metrics[2] = new Metric("D", "B", "C");
        }

        /// <summary>
        /// Tests construction with null parameters.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestNullConstructor1()
        {
            new Metric(null, "B", "C");
        }

        /// <summary>
        /// Tests construction with null parameters.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestNullConstructor2()
        {
            new Metric("A", null, "C");
        }

        /// <summary>
        /// Tests construction with null parameters.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestNullConstructor3()
        {
            new Metric("A", "B", null);
        }

        /// <summary>
        /// Tests construction with empty parameters.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestEmptyConstructor1()
        {
            new Metric("  ", "B", "C");
        }

        /// <summary>
        /// Tests construction with empty parameters.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestEmptyConstructor2()
        {
            new Metric("A", "  ", "C");
        }

        /// <summary>
        /// Tests construction with empty parameters.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestEmptyConstructor3()
        {
            new Metric("A", "B", "  ");
        }

        /// <summary>
        /// Tests the name property getter, relative to values from constructor in Setup.
        /// </summary>
        [Test]
        public void TestNameProperty()
        {
            Assert.AreEqual("A", metrics[0].Name);
            Assert.AreEqual("A", metrics[1].Name);
            Assert.AreEqual("D", metrics[2].Name);
        }

        /// <summary>
        /// Tests the display name property getter, relative to values from constructor in Setup.
        /// </summary>
        [Test]
        public void TestDisplayNameProperty()
        {
            Assert.AreEqual("B", metrics[0].DisplayName);
            Assert.AreEqual("C", metrics[1].DisplayName);
            Assert.AreEqual("B", metrics[2].DisplayName);
        }

        /// <summary>
        /// Tests the description property getter, relative to values from constructor in Setup.
        /// </summary>
        [Test]
        public void TestDescriptionProperty()
        {
            Assert.AreEqual("C", metrics[0].Description);
            Assert.AreEqual("B", metrics[1].Description);
            Assert.AreEqual("C", metrics[2].Description);
        }

        /// <summary>
        /// Verifies that the <c>Equals</c> method is looking at the <c>Name</c> property.
        /// </summary>
        [Test]
        public void TestEquals()
        {
            Assert.IsTrue(metrics[0].Equals(metrics[1]), "Should match only on name. 1");
            Assert.IsTrue(metrics[1].Equals(metrics[0]), "Should match only on name. 2");
            Assert.IsFalse(metrics[1].Equals(metrics[2]), "Mismatching names shouldn't match. 1");
            Assert.IsFalse(metrics[0].Equals(metrics[2]), "Mismatching names shouldn't match. 2");

            Assert.IsFalse(metrics[0].Equals(null), "Null equality always false.");
        }

        /// <summary>
        /// Verifies that the <c>GetHashCode</c> method is based on the <c>Name</c> property.
        /// </summary>
        [Test]
        public void TestHash()
        {
            Assert.AreEqual(metrics[0].GetHashCode(), metrics[1].GetHashCode(), "Should match only on name. 1");
            Assert.IsFalse(metrics[1].GetHashCode() == metrics[2].GetHashCode(), "Mismatching names shouldn't match.");
        }
    }
}