/*
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER, TCSDESIGNER
 * Version: 2.0b
 */


using System;
using System.Collections;

namespace TopCoder.File.Statistics
{
    /// <summary>
    /// <c>IArgumentHandler</c> is an interface added in version 2.0b to represent an entity that could potentially
    /// require arguments from the command line. Current the supported entities (that can potentially use arguments
    /// support) are <c>FileStatistics</c>, <c>IFileProcessor</c>, <c>IFileMatcher</c> and <c>IStatsReporting</c>.
    /// </summary>
    /// <remarks>
    /// The procedure works in two phases. In the first step the handler proposes the options that it supports to the
    /// command line interface, where some simple validation is performed. In the second step the arguments are
    /// received at the handler, where they are applied after possibly undergoing further validation.
    /// </remarks>
    public interface IArgumentHandler
    {
        /// <summary>
        /// This interface gives the handler the opportunity to present the options supported, in the form of an
        /// <c>ArgumentOption</c> list. This data is used to parse the command line and perform initial validation.
        /// </summary>
        /// <returns>An <c>IList</c> of non-null values, all of type <c>ArgumentOption</c>. Possibly empty.</returns>
        IList ProposeOptions();
        
        /// <summary>
        /// This method receives the argument from the command line. The key of the map is the switch name
        /// (<c>string</c>) and the value of the map is the argument value (also a string). If a switch is not included
        /// in the command line it should not show up in the map. If the switch is included in the command line without
        /// any argument values an empty string ("") is provided as the argument value.               
        /// </summary>
        /// <remarks>
        /// The interface only receives the arguments it has proposed.
        /// </remarks>
        /// <param name="arguments">A map of those switches (keys, strings) that were reported as supported in
        /// <c>ProposeOptions</c> and their argument values (as map values).</param>
        /// <exception cref="ArgumentNullException">If <c>arguments</c> is null.</exception>
        /// <exception cref="ArgumentException">If any value in the map is not a string. This is not a requirement,
        /// but allowed, it might make sense to only check the values actually used.</exception>
        /// <exception cref="InvalidArgumentException">If some argument is not valid, in the sense of invalid data
        /// provided by the user.</exception>
        void HandleArguments(IDictionary arguments);
    }
}
