/*
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER, TCSDESIGNER
 * Version: 2.0b
 */


using System;
using System.Collections;
using System.IO;

namespace TopCoder.File.Statistics
{
    /// <summary>
    /// <c>IFileMatcher</c> defines the contract to match a certain file (in order for an attaching
    /// <c>IFileProcessor</c> to process the file).
    /// </summary>
    /// <remarks>
    /// Implementations are advised to be stateless and thread-safe. But it's not a requirement. Also, if an
    /// implementation needs to be created from an XML configuration, it should have a public constructor accepting
    /// a configuration manager namespace and a property name prefix, to which suffixes for specific configuration
    /// arguments are appended.
    /// </remarks>
    public interface IFileMatcher
    {
        /// <summary>
        /// This is used to determine if the file can/will be processed by the processor that this matcher is
        /// associated with.
        /// </summary>
        /// <param name="file">The file to match.</param>
        /// <exception cref="ArgumentNullException">If <c>file</c> is null.</exception>
        /// <exception cref="IOException">If an IO operation fails.</exception>
        /// <exception cref="FileMatchingException">Other error conditions.</exception>
        /// <returns><c>true</c> if the file matches, <c>false</c> otherwise.</returns>
        bool Match(FileInfo file);
    }
}