/*
Copyright (c)2005, TopCoder, Inc. All rights reserved
Author Cucu
*/
using NUnit.Framework;
using TopCoder.File.Statistics;

using System;
using System.IO;
using System.Text.RegularExpressions;
using System.Collections;
using TopCoder.Util.ConfigurationManager;
using TopCoder.File.Statistics.AccuracyTests;
using TopCoder.File.Statistics.Reporting;

namespace TopCoder.File.Statistics.AccuracyTests
{
    /// <summary>
    /// Accuracy tests for CommandLine
    /// </summary>
    [TestFixture]
    public class CommandLineAccuracyTest
    {       

        /// <summary>
        /// Test running without parameters
        /// </summary>
        [Test]
        public void TestWithoutParam() 
        {
            Run(new string[] {}, "withoutParams.txt");
        }

        /// <summary>
        /// Test running with just a file
        /// </summary>
        [Test]
        public void Test1() 
        {
            Run(new string[] {"-c:" + TestHelper.FILE_STATISTICS_CONFIG, 
                                 "-n:" + TestHelper.FILE_STATISTICS_NAMESPACE, 
                                 TestHelper.TXT_FILE}, "result1.txt");
        }

        /// <summary>
        /// Test running with two files using a base directory
        /// </summary>
        [Test]
        public void Test2() 
        {
            Run(new string[] {"-c:" + TestHelper.FILE_STATISTICS_CONFIG,
                                 "-n:" + TestHelper.FILE_STATISTICS_NAMESPACE, 
                                 @"-b:..\..\test_files\AccuracyTests\files\",
                                "textFile1.txt",
                                "lyric.txt"}, "result2.txt");
        }

        /// <summary>
        /// Test running with a directory but not recursive
        /// </summary>
        [Test]
        public void Test3() 
        {
            Run(new string[] {"-c:" + TestHelper.FILE_STATISTICS_CONFIG,
                                 "-n:" + TestHelper.FILE_STATISTICS_NAMESPACE, 
                                 @"..\..\test_files\AccuracyTests\files\"}, "result3.txt");
        }

        /// <summary>
        /// Test running with a directory recursive
        /// </summary>
        [Test]
        public void Test4() 
        {
            Run(new string[] {"-c:" + TestHelper.FILE_STATISTICS_CONFIG,
                                 "-n:" + TestHelper.FILE_STATISTICS_NAMESPACE, 
                                 "-r",
                                 @"..\..\test_files\AccuracyTests\files\"}, "result4.txt");

        }

        /// <summary>
        /// Test running with output directed to a file
        /// </summary>
        [Test]
        public void Test5() 
        {           
            Run(new string[] {"-c:" + TestHelper.FILE_STATISTICS_CONFIG, 
                                 "-n:" + TestHelper.FILE_STATISTICS_NAMESPACE, 
                                 @"-o:..\..\test_files\AccuracyTests\CommandLineResults\result5.out",
                                 TestHelper.TXT_FILE}, "result5.txt");
        }

        /// <summary>
        /// Run from the command line, redirecting the output to the specified filename
        /// </summary>
        /// <param name="parameters">Parameters used to call the command line</param>
        /// <param name="filename">file to save the output</param>
        private void Run(string []parameters, string filename) 
        {
            TestHelper.UnloadConfig(TestHelper.FILE_STATISTICS_CONFIG);

            string descr = "Running application with parameters: ";
            foreach (string s in parameters) 
            {
                descr = descr + " " + s;
            }

            TextWriter previous = Console.Out;
            try 
            {           
                StreamWriter sw = new StreamWriter(@"..\..\test_files\AccuracyTests\CommandLineResults\" + filename);
                Console.SetOut(sw);

                Console.WriteLine(descr);
                CommandLine.Main(parameters);
                sw.Close();
            } 
            finally 
            {
                
                Console.SetOut(previous);
            }

        }

    }    
}