/*
Copyright (c)2005, TopCoder, Inc. All rights reserved
Author Cucu
*/
using NUnit.Framework;
using TopCoder.File.Statistics;

using System.IO;
using System.Text.RegularExpressions;
using System.Collections;
using TopCoder.Util.ConfigurationManager;
using TopCoder.File.Statistics.AccuracyTests;
using TopCoder.File.Statistics.Reporting;

namespace TopCoder.File.Statistics.AccuracyTests
{
    /// <summary>
    /// Accuracy tests for ExtendedFileStatistics
    /// </summary>
    [TestFixture]
    public class ExtendedFileStatisticsAccuracyTest
    {       

        /// <summary>
        /// Object to be tested
        /// </summary>
        private ExtendedFileStatistics efs;

        /// <summary>
        /// Set up for each test.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            TestHelper.LoadConfig(TestHelper.FILE_STATISTICS_CONFIG);
            efs = new ExtendedFileStatistics(TestHelper.FILE_STATISTICS_NAMESPACE);
        }

        /// <summary>
        /// Tear down for each test.
        /// </summary>
        [TearDown]
        public void Teardown()
        {
            TestHelper.UnloadConfig(TestHelper.FILE_STATISTICS_CONFIG);
        }

        /// <summary>
        /// Test the ProcessDirectory method (non recursive) 
        /// </summary>
        [Test]
        public void TestProcessDirectory()
        {
            efs.ProcessDirectory(new DirectoryInfo(@"..\..\test_files\AccuracyTests\files"));

            // Save to a file for visual inspection
            TestHelper.SaveToFile(@"FileStatisticsResults\Extended1.xml", efs.Reporting.GenerateReport());                
        }

        /// <summary>
        /// Test the ProcessDirectory recursive method 
        /// </summary>
        [Test]
        public void TestProcessDirectoryRecursive()
        {
            efs.ProcessDirectory(new DirectoryInfo(@"..\..\test_files\AccuracyTests\files"),true);

            // Save to a file for visual inspection
            TestHelper.SaveToFile(@"FileStatisticsResults\Extended2.xml", efs.Reporting.GenerateReport());                
        }

        /// <summary>
        /// Test PrintReport to the console
        /// </summary>
        [Test]
        public void TestPrintReport()
        {
            efs.ProcessDirectory(new DirectoryInfo(@"..\..\test_files\AccuracyTests\files"));
            System.Console.WriteLine("------ PrintReport result ------");
            efs.PrintReport();
            System.Console.WriteLine("------ End of PrintReport result ------");
        }

        /// <summary>
        /// Test PrintReport to a file
        /// </summary>
        [Test]
        public void TestPrintReportToFile()
        {
            efs.ProcessDirectory(new DirectoryInfo(@"..\..\test_files\AccuracyTests\files"));
            efs.PrintReport(new FileInfo(@"..\..\test_files\AccuracyTests\FileStatisticsResults\Extended3.xml"));
        }

    }    
}