/*
Copyright (c)2005, TopCoder, Inc. All rights reserved
Author Cucu
*/
using TopCoder.File.Statistics.Matcher;
using TopCoder.File.Statistics.Processor.LineCounter;
using NUnit.Framework;
using TopCoder.File.Statistics;
using System.IO;

namespace TopCoder.File.Statistics.AccuracyTests.Processor.LineCounter
{
    /// <summary>
    /// Accuracy tests for TextLineCounter
    /// </summary>
    [TestFixture]
    public class CStyleLineCounterAccuracyTest
    {  
        /// <summary>
        /// Test the C Style line count against some files.
        /// </summary>
        [Test]
        public void TestMatch()
        {
            IFileProcessor fp = new CStyleLineCounter("c", new AnyFileMatcher());
       
            VerifyLineCount(fp, TestHelper.CS_FILE, 3);
            VerifyLineCount(fp, TestHelper.JAVA_FILE, 9);
            VerifyLineCount(fp, TestHelper.CPP_FILE, 36);
        }

        /// <summary>
        /// Verifies that the line count is correct for a c style file
        /// </summary>
        /// <param name="fp">IFileProcessor that will process the file</param>
        /// <param name="filename">file whose C style lines will be counted</param>
        /// <param name="expectedCount">expected number of lines</param>
        private void VerifyLineCount(IFileProcessor fp, string filename, int expectedCount) 
        {
            FileInfo file = new FileInfo (filename);
            Assert.AreEqual(expectedCount, fp.Process(file)[(Metric) fp.SupportedMetrics[0]], 
                " bad C Style line count for "  + filename);
        }
    
	}
}
