/*
Copyright (c)2005, TopCoder, Inc. All rights reserved
Author Cucu
*/
using TopCoder.File.Statistics.Matcher;
using TopCoder.File.Statistics.Reporting;
using TopCoder.File.Statistics.Processor.LineCounter;
using NUnit.Framework;
using TopCoder.File.Statistics;
using System.IO;

namespace TopCoder.File.Statistics.AccuracyTests.Reporting
{
    /// <summary>
    /// Accuracy tests for XmlStatsReporting
    /// </summary>
    [TestFixture]
    public class XmlLStatsReportingAccuracyTest
    {       
        /// <summary>
        /// Test creating an empty report and saving it so it can be visually inspected.
        /// </summary>
        [Test]
        public void TestEmptyReport()
        {
            XmlStatsReporting bsr = new XmlStatsReporting();
            CreateReport(bsr, new string[]{},@"..\..\test_files\AccuracyTests\ReportResults\EmptyReport.xml");
        }

        /// <summary>
        /// Test creating a report of a file and saving it so it can be visually inspected.
        /// </summary>
        [Test]
        public void TestReport1()
        {
            XmlStatsReporting bsr = new XmlStatsReporting();
            CreateReport(bsr, new string[]{TestHelper.CS_FILE},@"..\..\test_files\AccuracyTests\ReportResults\Report1.xml");
        }

        /// <summary>
        /// Test creating a report of some files and saving it so it can be visually inspected.
        /// </summary>
        [Test]
        public void TestReport2()
        {
            XmlStatsReporting bsr = new XmlStatsReporting();
            CreateReport(bsr, new string[]{TestHelper.CS_FILE, TestHelper.JAVA_FILE, TestHelper.CPP_FILE, TestHelper.CPP_FILE_ANOTHER_DIR},
                    @"..\..\test_files\AccuracyTests\ReportResults\Report2.xml");
        }

        /// <summary>
        /// Test doing a reset to have a base directory.
        /// </summary>
        [Test]
        public void TestReset()
        {
            XmlStatsReporting bsr = new XmlStatsReporting();
            bsr.Reset(new DirectoryInfo(@"..\..\test_files\AccuracyTests\files\"));

            CreateReport(bsr, new string[]{TestHelper.CS_FILE, TestHelper.JAVA_FILE, TestHelper.CPP_FILE, TestHelper.CPP_FILE_ANOTHER_DIR},
                @"..\..\test_files\AccuracyTests\ReportResults\Reset.xml");
        }

        /// <summary>
        /// Test Stylesheet
        /// </summary>
        [Test]
        public void TestStylesheet()
        {
            XmlStatsReporting bsr = new XmlStatsReporting(new FileInfo(@"..\..\test_files\reporting.xsl"));

            CreateReport(bsr, new string[]{TestHelper.CS_FILE, TestHelper.JAVA_FILE, TestHelper.CPP_FILE, TestHelper.CPP_FILE_ANOTHER_DIR},
                @"..\..\test_files\AccuracyTests\ReportResults\ReportingWithStylesheet.html");
        }

        /// <summary>
        /// Creata a report with XMLStatsReporting and outputs it to a file
        /// </summary>
        /// <param name="files">files to be processed</param>
        /// <param name="outputFileName">name of the file where the report is written</param>
        private void CreateReport(XmlStatsReporting bsr, string []files, string outputFileName) 
        {
            IFileProcessor fp = new CStyleLineCounter("Source File", new AnyFileMatcher());

            foreach (string filename in files) 
            {
                StatsResult sr = new StatsResult();
                FileInfo fi = new FileInfo(filename);
                sr.Aggregate(fp.Process(fi));
                bsr.Aggregate(fp, fi, sr);
            }

            StreamWriter sw = new StreamWriter(outputFileName);
            sw.Write(bsr.GenerateReport());
            sw.Close();
        }

    }    
}