/*
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER, TCSDESIGNER
 * Version: 2.0b
 */

using System;
using System.IO;
using System.Reflection;
using TopCoder.Util.ConfigurationManager;
using NUnit.Framework;

namespace TopCoder.File.Statistics.Processor
{
    /// <summary>
    /// This class is a base for verifying basic functionality of <c>FileProcessorBase</c> implementations, based on
    /// <c>TestContract</c>.
    /// </summary>
    [TestFixture]
    public abstract class TestFileProcessorBase : TestContract
    {
        /// <summary>
        /// Stores two file processors (one from config, one from direct parameters) used for later tests.
        /// </summary>
        protected IFileProcessor[] procs = new IFileProcessor[5];

        /// <summary>
        /// Sets up the processors used in testing.
        /// </summary>
        [SetUp]
        public virtual void Setup()
        {
            procs[0] = (IFileProcessor) Constructor("procalias", new Matcher.TextFileMatcher());
            procs[1] = (IFileProcessor) Constructor("procalias", new Matcher.TextFileMatcher(), false);
            procs[2] = (IFileProcessor) Constructor("procalias", new Matcher.TextFileMatcher(), true);
            procs[3] = (IFileProcessor) Constructor("TopCoder.File.Statistics", "6.");
            procs[4] = (IFileProcessor) Constructor("TopCoder.File.Statistics", "16.");
        }

        /// <summary>
        /// Tests the direct (alias, matcher) constructor with null parameters.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestNullDirectConstructor1()
        {
            Constructor(null, new Matcher.AnyFileMatcher());
        }

        /// <summary>
        /// Tests the direct (alias, matcher) constructor with an empty alias.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestEmptyAliasDirectConstructor()
        {
            Constructor("  ", new Matcher.AnyFileMatcher());
        }

        /// <summary>
        /// Tests the alias property.
        /// </summary>
        [Test]
        public void TestAliasProperty()
        {
            for (int i = 0; i < 2; i++)
            {
                Assert.AreEqual("procalias", procs[i].Alias, "Wrong alias for processor " + i);
            }
        }

        /// <summary>
        /// Tests the file matcher.
        /// </summary>
        [Test]
        public void TestMatcherProperty()
        {
            for (int i = 0; i < 2; i++)
            {
                Assert.AreEqual(typeof(Matcher.TextFileMatcher), procs[i].FileMatcher.GetType(),
                    "Wrong matcher type for processor " + i);
            }
        }

        /// <summary>
        /// Tests processing a null file.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestProcessNull()
        {
            procs[0].Process(null);
        }
        
        /// <summary>
        /// A short helper extracting the lines from processing.
        /// </summary>
        /// <param name="index">The processor index to check.</param>
        /// <param name="filename">The filename.</param>
        /// <returns>The resulting number of lines.</returns>
        public long CountLines(int index, string filename)
        {
            StatsResult res = procs[index].Process(new FileInfo(filename));
            return res[(Metric) procs[index].SupportedMetrics[0]];
        }
    }
}
