using System;
using System.Collections;
using System.IO;
using System.Text.RegularExpressions;
using NUnit.Framework;
using TopCoder.File.Statistics.Reporting;
using TopCoder.File.Statistics.Matcher;

namespace TopCoder.File.Statistics.StressTests
{
	[TestFixture]
	public class MatchersTests
	{

		private ExtendedFileStatistics statistics;

		private int numProcessors = 10;

		private string pathToFile = @"..\..\test_files\stressdir";

		private int levels = 1;

		private int childrenAtLevel = 1;

		private int filesInNode = 1000;

		private string STANDARD_FILE_CONTENT = "....";

		private IList files = new ArrayList();
		private IList folders = new ArrayList();

		[SetUp]
		public void SetUp()
		{
			// create a file statistics instance
			statistics = new ExtendedFileStatistics();

			//assign the report object
			statistics.Reporting = new BasicStatsReporting();

			int k = 0;
			for(int i = 0; i < numProcessors; ++i)
			{
				// create file matchers
				IFileMatcher matcher = new AnyFileMatcher();
				statistics.AddFileProcessor(new NopProcessor("NOP" + k++, matcher));

				matcher = new ExtensionMatcher("txt");
				statistics.AddFileProcessor(new NopProcessor("NOP" + k++, matcher));

				matcher = new RegexMatcher(new Regex("pattern"));
				statistics.AddFileProcessor(new NopProcessor("NOP" + k++, matcher));

				matcher = new TextFileMatcher();
				statistics.AddFileProcessor(new NopProcessor("NOP" + k++, matcher));

				matcher = new BinaryFileMatcher();
				statistics.AddFileProcessor(new NopProcessor("NOP" + k++, matcher));

				matcher = new TypeNameMatcher("Java");
				statistics.AddFileProcessor(new NopProcessor("NOP" + k++, matcher));

				matcher = new MimeMatcher("text/plain");
				statistics.AddFileProcessor(new NopProcessor("NOP" + k++, matcher));
			}
			StreamReader reader = new StreamReader(System.IO.File.OpenRead(@"..\..\test_files\stresstests\standard_file.txt"));
			STANDARD_FILE_CONTENT = reader.ReadToEnd();
			reader.Close();
			DirectoryInfo root = new DirectoryInfo(@"..\..\test_files\stressdir");
			folders.Add(root.FullName);
			generate(0, root);
		}

		private void generate(int levelNum, DirectoryInfo directory)
		{
			if (levelNum != 0)
			{
				for(int i = 0; i < filesInNode; ++i)
				{
					string filename = String.Format(@"{0}\{1}.txt", directory.FullName, i);
					FileStream file = System.IO.File.Create(filename);
					StreamWriter writer = new StreamWriter(file);
					writer.Write(STANDARD_FILE_CONTENT);
					writer.Close();
					files.Add(filename);
				}
			}
			if (levelNum < levels)
			{
				for(int i = 0; i < childrenAtLevel; ++i)
				{
					DirectoryInfo subDir = directory.CreateSubdirectory(String.Format("d{0}", i));
					folders.Add(subDir.FullName);
					generate(levelNum + 1, subDir);
				}
			}
		}

		[TearDown]
		public void TearDown()
		{
			for(int i = 0; i < files.Count; ++i)
			{
				string name = (string)files[i];
				FileInfo file = new FileInfo(name);
				file.Delete();
			}
			for(int i = folders.Count - 1; i >= 0; --i)
			{
				string name = (string)folders[i];
				DirectoryInfo dir = new DirectoryInfo(name);
				dir.Delete();
			}
		}

		/// <summary>
		/// Attempts to process one file with a large number of matchers.
		/// The aim is to test the algorithm used to process the files in a situation
		/// where there are many files and many matchers.
		/// </summary>
		[Test]
		public void TestManyMatchers()
		{
			DateTime start = DateTime.Now;
			statistics.ProcessDirectory(new DirectoryInfo(pathToFile), false);
			TimeSpan duration = DateTime.Now - start;
			Console.WriteLine("Working with a large number of matchers took {0} ms", duration.TotalMilliseconds);
		}
	}
}